@@ -1,8 +1,9 @@
 #!/usr/bin/env python
 """ Provides functions to load and write models, media definitions and the manual annotation table
 
-Depending on the application the model needs to be loaded with cobra (memote)
-or with libSBML (activation of groups). The media definitions are denoted in a csv within the data folder of this repository, thus the functions will only work if the user clones the repository. The manual_annotations table has to follow the specific layout given in the data folder in order to work with this module.
+Depending on the application the model needs to be loaded with cobra (memote) or with libSBML (activation of groups). 
+The media definitions are denoted in a csv within the data folder of this repository, thus the functions will only work if the user clones the repository. 
+The manual_annotations table has to follow the specific layout given in the data folder in order to work with this module.
 """
 
 import cobra
@@ -187,6 +188,31 @@ def load_manual_gapfill(tablepath: str='data/manual_curation.xlsx' , sheet_name:
     return man_gapf
 
 
+def parse_dict_to_dataframe(str2list: dict) -> pd.DataFrame:
+    """| Parses dictionary of form {str: list} & 
+       | Transforms it into a table with a column containing the strings and a column containing the lists
+
+    Args:
+        str2list (dict): Dictionary mapping strings to lists
+
+    Returns:
+        pd.DataFrame: Table with column containing the strings and column containing the lists
+    """
+    # Get max number of list length
+    max_len_of_list = max(map(len, str2list.values()))
+
+    # Fill lists with None until all lists have the same size -> Required for pd.DataFrame
+    for key in str2list:
+       current_list = str2list.get(key)
+       while len(current_list) != max_len_of_list:
+          str2list.get(key).append(None)
+
+    df = pd.DataFrame.from_dict(str2list).stack().T.reset_index()
+    df = df.drop('level_0', axis=1)
+    
+    return df
+
+
 def write_to_file(model: libModel, new_filename: str):
     """Writes modified model to new file
 
@@ -451,7 +477,6 @@ def save_user_input(configpath: str) -> dict[str: str]:
                     not_valid = False
                 else:
                     print('File does not exist. Please enter a valid file path')
-        
             user_input['memote'] = click.confirm('Do you want to run MEMOTE (takes some time)?')    
             user_input['modelseed'] = click.confirm('Do you want to compare your model entities to the ModelSEED database?')
         
@@ -481,10 +506,12 @@ def save_user_input(configpath: str) -> dict[str: str]:
                 else:
                     user_input['model_out'] = click.prompt('Enter path and filename to where to save the modified model')
                 
-                user_input['gapfill_model'] = click.confirm('Do you want to fill gaps in your model?')
+                gapfill_model = click.confirm('Do you want to fill gaps in your model?')
+                user_input['gapfill_model'] = gapfill_model
                 
-                if not gap_analysis:
-                    user_input['gap_analysis_file'] = click.prompt('Enter path to Excel file with which gaps should be filled')
+                if gapfill_model:
+                    if not gap_analysis:
+                        user_input['gap_analysis_file'] = click.prompt('Enter path to Excel file with which gaps should be filled')
                 
                 user_input['keggpathways'] = click.confirm('Do you want to add KEGG Pathways?')
                     
@@ -516,9 +543,13 @@ def save_user_input(configpath: str) -> dict[str: str]:
                     protein_fasta = click.prompt('If possible, provide the path to your Protein FASTA file used for CarveMe')
                     user_input['protein_fasta'] = protein_fasta
                     
+                biomass = click.confirm('Do you want to check & normalise the biomass function(s)?')
+                user_input['biomass'] = biomass
+                    
             else:
                 user_input['keggpathways'] = False
                 user_input['polish'] = False
+                user_input['biomass'] = False
                 user_input['sboterms'] = False
                 user_input['charge_corr'] = False
                 user_input['gapfill_model'] = False
